#ifndef KADU_SOUND_H
#define KADU_SOUND_H

#include <qobject.h>
#include <qstring.h>
#include <qdatetime.h>
#include <qstringlist.h>
#include <qmap.h>

#include "config_file.h"
#include "modules.h"
#include "gadu.h"
#include "userlist.h"
#include "misc.h"

class SoundSlots : public QObject
{
	Q_OBJECT
	private:
		int muteitem;
		QMap<QString, QString> soundfiles;
		QStringList soundNames;
		QStringList soundTexts;

	private slots:
		void soundPlayer(bool value, bool toolbarChanged=false);
		void onCreateConfigDialog();
		void onApplyConfigDialog();
		void chooseSoundTheme(const QString& string);
		void chooseSoundFile();
		void clearSoundFile();
		void testSoundFile();
		void selectedPaths(const QStringList& paths);
		void muteUnmuteSounds();
		void testSamplePlaying();
		void testSampleRecording();
		void testFullDuplex();

	public:
		SoundSlots(QObject *parent=0, const char *name=0);
		~SoundSlots();
};

/**
	Uoglniony deskryptor urzdzenia dwikowego.
**/
typedef void* SoundDevice;

class SoundManager : public Themes
{
    Q_OBJECT
	private:
		QTime lastsoundtime;
		bool mute;

	private slots:
		void newChat(const UinsList &senders, const QString& msg, time_t time);
		void newMessage(const UinsList &senders, const QString& msg, time_t time, bool &grab);
		void connectionError(const QString &message);
		void userChangedStatusToAvailable(const UserListElement &ule);
		void userChangedStatusToBusy(const UserListElement &ule);
		void userChangedStatusToInvisible(const UserListElement &ule);
		void userChangedStatusToNotAvailable(const UserListElement &ule);
		/* from i ule s ignorowane, message wskazuje na plik z dwikiem do odtworzenia
		 * jeeli message==QString::null, to odtwarzany jest standardowy dwik dla tego typu 
		 * jeeli mapa jest!=NULL brane s z niej nastpujce wartoci:
		 *		"Force"           - bool (wymuszenie odtwarzania mimo wyciszenia)
		 */
		void message(const QString &from, const QString &message, const QMap<QString, QVariant> *parameters, const UserListElement *ule);

	public slots:
		void play(const QString &path, bool force=false);
		void setMute(const bool& enable);

	public:
		SoundManager(const QString& name, const QString& configname);
		~SoundManager();
		bool isMuted();
		int timeAfterLastSound();
		/**
			Otwiera urzdzenie dwikowe do operacji
			odtwarzania i nagrywania sampli.
			Niektre implementacje pozwalaj na otwarcie
			wielu niezalenie dziaajcych "pocze"
			z urzdzeniami. Wystarczy wtedy kilkukrotnie
			wywoa t metod.
			Emituje sygna openDeviceImpl() w celu
			przekazania dania do konkrentego moduu
			dwikowego.
			@param sample_rate sample rate - np. 8000 lub 48000
			@channels ilo kanaw: 1 - mono, 2 - stereo
			@return uoglniony deskryptor urzdzenia lub NULL jeli otwarcie si nie powiodo.
		**/
		SoundDevice openDevice(int sample_rate, int channels = 1);
		/**
			Zamyka urzdzenie dwikowe otwarte za pomoc
			metody openDevice().
			Niektre implementacje pozwalaj na otwarcie
			wielu niezalenie dziaajcych "pocze"
			z urzdzeniami. Kade otwarte poczenie naley
			zamkn za pomoc tej metody.
			Emituje sygna closeDeviceImpl() w celu
			przekazania dania do konkrentego moduu
			dwikowego.
			@param device uoglniony deskryptor urzdzenia.
		**/
		void closeDevice(SoundDevice device);
		/**
			Odtwarza prbk dwikow. Operacja blokujca.
			Moe by wywoana z innego wtku (a nawet powinna).
			Emituje sygna playSampleImpl() w celu
			przekazania dania do konkrentego moduu
			dwikowego.
			@param device uoglniony deskryptor urzdzenia
			@data wskanik do danych sampla
			@length dugo danych sampla
			@return true jeli odtwarzanie zakoczyo si powodzeniem.
		**/
		bool playSample(SoundDevice device, const int16_t* data, int length);
		/**
			Nagrywa prbk dwikow. Operacja blokujca.
			Moe by wywoana z innego wtku (a nawet powinna).
			Emituje sygna recordSampleImpl() w celu
			przekazania dania do konkrentego moduu
			dwikowego.
			@param device uoglniony deskryptor urzdzenia
			@data wskanik na bufor dla danych sampla
			@length dugo sampla do nagrania (wielko bufora)
			@return true jeli nagrywanie zakoczyo si powodzeniem.
		**/
		bool recordSample(SoundDevice device, int16_t* data, int length);

	signals:
		void playSound(const QString &sound, bool volCntrl, double vol);
		/**
			Pod ten sygna powinien podpi si modu
			dwikowy jeli obsuguje funkcj odtwarzania
			prbki dwikowej.
			Wyemitowanie sygnau oznacza danie
			otwarcia urzdzenia dwikowego do operacji
			odtwarzania i nagrywania sampli.
			@param sample_rate sample rate - np. 8000 lub 48000
			@channels ilo kanaw: 1 - mono, 2 - stereo
			@device zwrcony uoglniony deskryptor urzdzenia lub NULL jeli otwarcie si nie powiodo.
		**/
		void openDeviceImpl(int sample_rate, int channels, SoundDevice& device);
		/**
			Pod ten sygna powinien podpi si modu
			dwikowy jeli obsuguje funkcj odtwarzania
			prbki dwikowej.
			Wyemitowanie sygnau oznacza danie
			Zamknicia urzdzenia dwikowego otwartegp za pomoc
			metody openDevice().
			@param device uoglniony deskryptor urzdzenia.
		**/
		void closeDeviceImpl(SoundDevice device);
		/**
			Pod ten sygna powinien podpi si modu
			dwikowy jeli obsuguje funkcj odtwarzania
			prbki dwikowej.
			Wyemitowanie sygnau oznacza danie
			odtworzenia prbki dwikowej.
			Modu powinien odda sterowanie dopiero po
			odtworzeniu prbki.
			Sygna zazwyczaj bdzie emitowany z innego
			wtku i slot musi by do tego przystosowany.
			@param device uoglniony deskryptor urzdzenia
			@data wskanik do danych sampla
			@length dugo danych sampla
			@result zwrcony rezultat operacji - true jeli odtwarzanie zakoczyo si powodzeniem.
		**/
		void playSampleImpl(SoundDevice device, const int16_t* data, int length, bool& result);
		/**
			Pod ten sygna powinien podpi si modu
			dwikowy jeli obsuguje funkcj odtwarzania
			prbki dwikowej.
			Wyemitowanie sygnau oznacza danie
			nagrania prbki dwikowej.
			Modu powinien odda sterowanie dopiero po
			nagraniu prbki.
			Sygna zazwyczaj bdzie emitowany z innego
			wtku i slot musi by do tego przystosowany.
			@param device uoglniony deskryptor urzdzenia
			@data wskanik na bufor dla danych sampla
			@length dugo sampla do nagrania (wielko bufora)
			@result zwrcony rezultat operacji - true jeli nagrywanie zakoczyo si powodzeniem.
		**/
		void recordSampleImpl(SoundDevice device, int16_t* data, int length, bool& result);		
};

extern SoundManager* sound_manager;

#endif
